/*
 * rand_delayed_exec.c
 *
 * R. Schulze <rob@rob-schulze.de> 2009
 * http://www.rob-schulze.de/publikationen/frickelworx/rand_delayed_exec/
 */

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#define MAX_MS 1000

void usage(void);

int main(int argc, char *argv[], char *envp[])
{
if(argc<4)
	{
	usage();
	return 0;
	}

int msecs=atoi(argv[1]);
int prob=atoi(argv[2]);

if( (msecs>=MAX_MS || msecs<1) || (prob>=100 || prob<1) || (*argv[3]==0) )
	{
	usage();
	return 2;
	}

srand(getpid()*time(NULL));
float rnd=(rand()/(RAND_MAX+1.0));

#ifdef DEBUG
fprintf(stderr,"rnd: %f prob:%d(%f)\n",rnd,prob,(prob/100.0));
#endif

if(rnd<(prob/100.0))
	{
	#ifdef DEBUG
	fprintf(stderr,"delaying execution for %d msec\n",msecs);
	#endif
	struct timespec ts;
	ts.tv_sec=0;
	ts.tv_nsec=1000000*msecs;
	nanosleep(&ts,NULL);
	}

execv(argv[3],argv+3);
perror("unable to exec");
return 1;
}

void usage(void)
{
fprintf(stderr,"usage:\trand_delayed_exec <ms> <prob> <cmd [args]>\n\n\
	defers execution of cmd for ms milliseconds\n\
	by a probability of prob percent\n\
	\n\
input rules:\n\
	0 < ms < %d\n\
	0 < prob < 100\n",MAX_MS);
}




